#!/bin/bash

# mapWOC
#
# Description: Allow and dump traffic for native HCs (on vlan devices).
#
# Authors:
# Felix Wolfsteller <felix.wolfsteller@intevation.de>
#
# Copyright:
# Copyright (C) 2011 Bundesamt für Sicherheit in der Informationstechnik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2,
# or, at your option, any later version as published by the Free
# Software Foundation
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

# Parse args: <vlan> <survivor_file> <dump_file>
if [ $# -ne 3 ] ; then
  # Need exactly 3 arguments.
  exit 2
fi

function cleanup {
  iptables -D INPUT  -i $vlan -j ACCEPT
  iptables -D OUTPUT -o $vlan -j ACCEPT
  iptables -A INPUT  -i $vlan -j REJECT
  iptables -A OUTPUT -o $vlan -j REJECT
  ifdown $vlan
  #kill 0
  exit 0
}

vlan="$1"
survivor_file="$2"
dump_file="$3"

# Check that survivor_file and dump_file are files, really.
if [ ! -f $survivor_file ] ; then
  exit 1
fi

if [ ! -f $dump_file ] ; then
  exit 1
fi


# White-list vlan argument.

case $vlan in
 vlan101 | vlan102 | vlan103 | vlan104)
   # VLAN argument fine.
   ;;
 * )
   # VLAN not in whitelist.
   exit 3
   ;;
esac 

# Install signal handlers.
trap cleanup INT TERM HUP KILL

# Forbid traffic on <vlan>.
iptables -D INPUT  -i $vlan -j ACCEPT
iptables -D OUTPUT -o $vlan -j ACCEPT
iptables -A INPUT  -i $vlan -j REJECT
iptables -A OUTPUT -o $vlan -j REJECT

# Bring up the <vlan> device.
ifup $vlan

# Start tcpdump on <vlan>.
tcpdump -i $vlan -w $dump_file &

# Allow traffic on <vlan>.
iptables -D INPUT  -i $vlan -j REJECT
iptables -D OUTPUT -o $vlan -j REJECT
iptables -A INPUT  -i $vlan -j ACCEPT
iptables -A OUTPUT -o $vlan -j ACCEPT

while [ -f $survivor_file ] ; do
  sleep 1
done

# Wait for children/signal.
#wait

#kill 0

cleanup

# Guard. Not reached.
exit 0
