# -*- coding: utf-8 -*-

# mapWOC
#
# Description: Utils wrt gnutls.
#
# Authors:
# Felix Wolfsteller <felix.wolfsteller@intevation.de>
#
# Copyright:
# Copyright (C) 2011 Bundesamt für Sicherheit in der Informationstechnik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2,
# or, at your option, any later version as published by the Free
# Software Foundation
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

from ctypes import *
from os import path

from gnutls.constants import *
from gnutls.crypto import *
from gnutls.errors import *
from gnutls.interfaces.twisted import X509Credentials
from gnutls.library.functions import gnutls_x509_crt_get_fingerprint

def load_cred(certs_path):
    """
    Loads credentials from given path.
    """
    cert_file = path.join(certs_path, "cert.pem")
    key_file = path.join(certs_path, "key.pem")
    ca_file = path.join(certs_path, "ca-cert.pem")
    cert = X509Certificate(open(cert_file).read())
    ca = X509Certificate(open(ca_file).read())
    key = X509PrivateKey(open(key_file).read())
    cred = X509Credentials(cert, key, [ca])
    cred.verify_peer = True
    cred.session_params.compressions = (COMP_LZO, COMP_DEFLATE, COMP_NULL)
    return cred


def get_cert_fingerpring(cert):
    """
    Get fingerprint of given cert.
    """
    size = c_size_t(24)
    dname = create_string_buffer(size.value)
    try:
        gnutls_x509_crt_get_fingerprint(cert._c_object, 4, dname, byref(size))
        answer = ""
        for c in dname:
            answer += '%02X:' % ord(c)
        return answer
    except:
        raise

def recvall(session):
    """
    Receive all what is to received from session, likely
    invalidating it and catching any issues occuring after first read.
    """
    BUFSZ = 1024
    received = ''
    buf = session.recv(BUFSZ)
    while buf:
        received += buf
        try:
            buf = session.recv(BUFSZ)
        except:
            buf = None
    return received
