# -*- coding: utf-8 -*-

# mapWOC
#
# Description: HC Data Type.
#
# Copyright:
# Copyright (C) 2011 Bundesamt für Sicherheit in der Informationstechnik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2,
# or, at your option, any later version as published by the Free
# Software Foundation
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

import ConfigParser

class HC(object):
    """Bundles information about a startable HoneyClient."""

    def __init__(self, io=None):
        """Read information about a honeyclient from a keyfile."""

        if io is not None:
            self.__from_io(io)
        else:
            self.name = None
            self.description = None
            self.hcsid = None
            self.active_browser = None
            self.installed_software = []
            self.os = None
            self.hctype = None
            self.ssid = None
            self.max_parallel = -1
        self.on_nodes = []


    def __from_io(self, io):
        """
        Fills fields from an object that implements readline (e.g. file).
        @param io: readline()-implementing object to be used with
                   rawconfigparser.
        """

        config = ConfigParser.RawConfigParser()
        config.read(io)
        if not config.has_section("hc"):
            raise Exception("HC File invalid (missing 'hc' section).")

        section = "hc"
        self.name = config.get(section, 'name')
        self.description = config.get(section, 'description')
        self.hcsid = config.get(section, 'hcsid')
        self.max_parallel = config.getint(section, 'max_parallel')
        self.active_browser = config.get(section, 'active_browser')
        self.installed_software = config.get(section, 'installed_software').split(';')
        try:
            self.os = config.get(section, 'os')
        except:
            self.os = "Unknown"
