# -*- coding: utf-8 -*-

# mapWOC
#
# Description: Helper functions to send mails
#
# Authors:
# Andre Heinecke <aheinecke@intevation.de>
#
# Copyright:
# Copyright (C) 2011 Bundesamt für Sicherheit in der Informationstechnik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2,
# or, at your option, any later version as published by the Free
# Software Foundation
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

import smtplib
import email
import email.Utils

#TODO localize
#TODO get more information in the mails.
#TODO write serious templates
TEMPLATE_FINISHED="""From: %(sender)s
To: %(reciever)s
Subject: mapWOC: %(scanname)s erfolgreich beendet.
MIME-Version: 1.0
Content-type: text/plain

%(scanname)s wurde erfolgreich beendet.

"""
TEMPLATE_ERROR="""From: %(sender)s
To: %(reciever)s
Subject: mapWOC: Error %(errornr)d
MIME-Version: 1.0
Content-type: text/plain

error %(errornr)d occured with the message %(errormessage)s

"""

class MailDispatcher(object):
    """ Helper class to send out mails. """

    def __init__(self, smtp_host, smtp_port, admin_mail, sender):
        if(smtp_port == None or
           smtp_host == None or
           admin_mail== None or
           sender == None):
            self.disabled = True
        else:
            self.disabled = False
        self.smtp_host = smtp_host
        self.smtp_port = smtp_port
        self.vars = {"reciever": admin_mail,
                     "sender" : sender }

    def send_scan_finished(self, **kw):
        """
        Send the scan_finished template.

        @param kw: Template variables. May not be Unicode
        """
        vars = self.vars.copy()
        vars.update(kw)
        self.send_mail(TEMPLATE_FINISHED % vars)

    def send_error(self, **kw):
        """
        Send the error template.

        @param kw: Template variables. May not be Unicode
        """
        vars = self.vars.copy()
        vars.update(kw)
        self.send_mail(TEMPLATE_ERROR % vars)

    def send_mail(self, raw_message):
        """
        Send a mail.

        @param raw_message: A correctly formatted mail.
        """
        if self.disabled: return
        msg = email.message_from_string(raw_message)
        sender = email.Utils.parseaddr(msg["From"])[1]
        recipients = [addr[1] for addr in
                        email.Utils.getaddresses(msg.get_all("To", [])
                                       + msg.get_all("Cc", []))]
        server = smtplib.SMTP(self.smtp_host, self.smtp_port)
        server.sendmail(sender, recipients, raw_message)
        server.quit()
