# -*- coding: utf-8 -*-

# mapWOC
#
# Description: SQLAlchemy/ORM-Mapper for ScanConfigurations.
#
# Authors:
# Felix Wolfsteller <felix.wolfsteller@intevation.de>
#
# Copyright:
# Copyright (C) 2011,2012 Bundesamt für Sicherheit in der Informationstechnik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2,
# or, at your option, any later version as published by the Free
# Software Foundation
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

from sqlalchemy import Table, Column, ForeignKey, Integer, String, Text, DateTime, Boolean
from sqlalchemy.orm import relationship, mapper

from mapwocgui.model.meta import Base, metadata

from mapwoc.lib.scanconfiguration import ScanConfiguration, VisitorConfiguration, VisitorBundle, ScanRun
from mapwoc.lib.url import URLList

scanconf_urllist = Table('scanconf_urllist', metadata,
                         Column('scanconfiguration_id',
                                ForeignKey('scanconfiguration.id')),
                         Column('urllist_id', Integer,
                                ForeignKey('urllist.id')))

visitorconf_visitorbundle = Table('visitorconf_visitorbundle', metadata,
                                  Column('visitorconfiguration_id', Integer,
                                       ForeignKey('visitorconfiguration.id')),
                                  Column('visitorbundle_id', Integer,
                                       ForeignKey('visitorbundle.id')))


visitorconf_table = Table('visitorconfiguration', metadata,
                          Column('id', Integer, primary_key=True),
                          Column('active_window', Integer, nullable=False),
                          Column('eventmap', Text),
                          Column('num_urls', Integer, nullable=True))

visitorbundle_table = Table('visitorbundle', metadata,
                            Column('id', Integer, primary_key=True),
                            Column('hcsid', Text, nullable=False),
                            Column('number', Integer, nullable=False),
                            Column('node', Text, nullable=False))

scanconf_table = Table('scanconfiguration', metadata,
                       Column('id', Integer, primary_key=True),
                       Column('name', Text, nullable=False),
                       Column('mode', String),
                       Column('comment', Text, nullable=True),
                       Column('description', Text, nullable=True),
                       Column('enable_vnc', Boolean, nullable=True),
                       Column('visitorconfiguration_id', None,
                              ForeignKey('visitorconfiguration.id')) )

scanrun_table = Table('scanrun', metadata,
                      Column('id', Integer, primary_key=True),
                      Column('scanconfiguration_id', None,
                             ForeignKey('scanconfiguration.id')),
                      Column('starttime', DateTime),
                      Column('stoptime', DateTime),
                      Column('status', Text))

mapper(VisitorBundle, visitorbundle_table)
mapper(VisitorConfiguration, visitorconf_table, properties={
                            'visitors':relationship(VisitorBundle,
                             secondary=visitorconf_visitorbundle)})
mapper(ScanConfiguration, scanconf_table, properties={
       'urlcollection':relationship(URLList, secondary=scanconf_urllist),
       'visitor_configuration':relationship(VisitorConfiguration,
                                            uselist=False),
       'scanruns':relationship(ScanRun,uselist=True)})

mapper(ScanRun, scanrun_table, properties={
       'scanconfiguration':relationship(ScanConfiguration)})
