# -*- coding: utf-8 -*-

# mapWOC
#
# Description: Visits, Results and ResultType Classes.
#
# Authors:
# Felix Wolfsteller <felix.wolfsteller@intevation.de>
# Andre Heinecke <aheinecke@intevation.de>
#
# Copyright:
# Copyright (C) 2011 Bundesamt für Sicherheit in der Informationstechnik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2,
# or, at your option, any later version as published by the Free
# Software Foundation
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

import datetime

"""
Map ResultTypes to severities and primary keys for db/ids.
TODO make the result types accessible by keyword not by string (ResultType.404
instead of "404").
"""
RESULT_TYPES={
        "unknown" : (999,1),
        "RegularTraffic" : (000,2),
        "404" : (100,3),
        "GoogleAnalyticsParser" : (500,4),
        "MissingTraffic" : (600,5),
        "WindowsExecutable" : (500,6),
        "GraphicsFound" : (50,7),
        "JSAnalyser" : (100,8),
        "204" : (99,9),
        "302" : (60,10),
        "UnknownProtocol": (001,11),
        "VirusFound" : (900,12),
        "Flash" : (200,13),
        "PDF" : (199,14),
        "MSOfficeDocument" : (210,15),
        "MACExecutable" : (211,16),
        }

def static_result_type_id(result_type_name):
    """Return primary key of a result type, 1 (unknown) if not found."""
    try:
        (_severity,pk) = RESULT_TYPES[result_type_name]
        return pk
    except:
        return 1

def static_result_type_severity(result_type_id):
    """
    Returns the severity of a ResultType, found by its id, searched linearly.
    @return: The result types severity or -1 if not found.
    """

    for (__name,(severity,rtid)) in RESULT_TYPES.iteritems():
        if rtid == result_type_id:
            return severity
    return -1

class ResultFile(object):
    """A File associated to a result."""
    def __init__(self):
        self.filepath = None
        self.filetype = None
        self.analysed_type = None
        self.mimetype = None
        self.sha1sum = None
        self.ssdeep = None

class ResultType(object):
    """Type of a result with associated severity."""
    def __init__(self, name, severity):
        self.type_name = name
        self.severity = severity

class Result(object):
    """A Result (single finding of a Visit."""

    def __init__(self, url=None, result_type=None, method='', date_time=None):
        """
        Creates a new result.

        @param url: The URL String that this Result relates to.
        @param result_type: The ResultType of this Result.
        @param method: The method string of this Result.
        @param date_time: Time of finding. Is set to datetime.now() if None.
        """
        self.url = url
        self.date = date_time
        self.result_type = result_type
        self.text = None
        self.ip = None
        self.iframe = None
        self.method = method
        self.result_file = None

class Visit(object):
    """A Visit of one or many URLs."""

    def __init__(self):
        self.hcsid = ""
        self.results = []
        self.node = ""
        self.date = None
        self.outgoing_ip = None
        self.filepath = None
        self.externalInfo = None
        self.urls = []
        self.ips = None
