# -*- coding: utf-8 -*-

# mapWOC
#
# Description: Scan Configuration Data Type.
#
# Authors:
# Felix Wolfsteller <felix.wolfsteller@intevation.de>
#
# Copyright:
# Copyright (C) 2011,2012 Bundesamt für Sicherheit in der Informationstechnik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2,
# or, at your option, any later version as published by the Free
# Software Foundation
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

import datetime

class VisitorBundle(object):
    """Triple (hcsid, number, node)."""

    def __init__(self, hcsid=None, number=None, node=None):
        self.hcsid = hcsid
        self.number = number
        self.node = node

class VisitorConfiguration(object):
    """Information valid for all visitors of a scan."""

    def __init__(self, timeout=100, num_urls=1):
        self.active_window = timeout
        self.num_urls = num_urls
        self.visitors = []
        self.eventmap = ""

class ScanConfiguration(object):
    """Information valid for a scan."""

    def __init__(self, name):
        self.name = name
        self.comment = None
        self.description = None
        self.visitor_configuration = None
        self.urlcollection = []
        self.enable_vnc = False

    def is_timed_scan(self):
        return self.visitor_configuration.active_window < 0

class TransportableSC(object):
    """
    TransportableSC object that can be used to transport the
    ScanConfiguration over a json connection.
    Do not forget to modify mapwoc_json if you modify TransportableSC.
    """

    def __init__(self, sc = None):
        """
        Inits a TransportableSC from a ScanConfiguration or another
        TransportableSC.
        @param sc: TransportableSC or ScanConfiguration to clone.
        """
        if sc:
            self.name = sc.name
            self.comment = sc.comment
            self.description = sc.description
            self.visitor_configuration = sc.visitor_configuration
            self.urllistIDs = [urllist.id for urllist in sc.urlcollection]
            self.id = sc.id
            self.enable_vnc = sc.enable_vnc

class ScanRun(object):
    """Execution of a Scan."""

    def __init__(self, scanconf=None, status=''):
        self.starttime = datetime.datetime.now()
        self.stoptime = None
        self.status = status
        self.scanconfiguration = scanconf
