# -*- coding: utf-8 -*-

# mapWOC
#
# Description: URL and URL Lists.
#
# Authors:
# Felix Wolfsteller <felix.wolfsteller@intevation.de>
#
# Copyright:
# Copyright (C) 2011 Bundesamt für Sicherheit in der Informationstechnik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2,
# or, at your option, any later version as published by the Free
# Software Foundation
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

from formencode import validators
from formencode.validators import CIDR
from datetime import datetime
import re


# Taken from django/core/validators.py
URLEXP_SHORT=(
r'(^https?://)' # http:// or https://
r'((?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+[A-Z]{2,6}\.?|' #domain...
r'localhost|' #localhost...
r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}))' # ...or ip
r'((?::\d+)?)' # optional port
r'((?:/?|[/?]\S+)$)') # Path

class URL(object):

    def __init__(self, url=None):
        """URL object.  Assign a value to url."""
        self.url = url


class URLList(object):

    def __init__(self, name):
        self.name = name
        self.urls = []
        self.comment = ''
        self.date = datetime.now()
        self.deleted = False

    def delete(self):
        self.deleted = True

def create_URL_List(name, urls, comment = None):
    """
    Create an urllist from a linefeed seperated list of urls.
    @param urls: String with linefeed separated list of urls.
    @param comment: Optional comment for URL-List.
    @return: Pair (validurls, invalidurls) where the first item is
             an URLList Instance and the second is a list of Strings.
    """
    import warnings
    warnings.warn(("lib.url.create_url_list is deprecated,"
                  "use lib.model.url insert list instead"),
                  category=DeprecationWarning)
    invalid_urls = []
    urllist = URLList(name)
    urllist.comment = comment or ''
    stripped_lines = [ line.strip() for line in urls.split() ]
    for line in stripped_lines:
        if not validate_URL(line):
            invalid_urls.append(line)
            continue
        urllist.urls.append(URL(line))
    urllist.numurls = len(urllist.urls)
    return (urllist,invalid_urls)

def separate_valid_urls(urls):
    """
    @param urls: String with linefeed separated list of urls.
    @return: Pair (validurls, invalidurls) where the first item is a list of
             strings of valid urls and the second is a list of invalid urls.
    """
    invalid_urls = []
    valid_urls = []
    stripped_lines = [ line.strip() for line in urls.split() ]
    for line in stripped_lines:
        if not validate_URL(line):
            invalid_urls.append(line)
        else:
            valid_urls.append(line)
    return (valid_urls,invalid_urls)

def validate_URL(urlstring):
    """
    Checks if the given string urlstring is a valid URL.
    @param urlstring: String with url.
    @return True if valid, False otherwise.
    """

    urlex = re.compile(URLEXP_SHORT, re.IGNORECASE)
    if urlex.match(urlstring):
        return True
    return False
