# -*- coding: utf-8 -*-

# mapWOC
#
# Description: Utility functions that can be useful everywhere.
#
# Authors:
# Andre Heinecke <aheinecke@intevation.de>
# Felix Wolfsteller <felix.wolfsteller@intevation.de>
#
# Copyright:
# Copyright (C) 2011 Bundesamt für Sicherheit in der Informationstechnik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2,
# or, at your option, any later version as published by the Free
# Software Foundation
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

import hashlib

def sha1sum(filepath):
    """
    Calculate the sha1sum of a file.
    @param filepath: Path the the file to hash.

    @return: The Hexdigest of the file.
    """
    fileHash = hashlib.sha1()
    with open(filepath, "rb") as hashFile:
        fileHash.update(hashFile.read())
    return fileHash.hexdigest()

def str_vacuum(string, max_length=20, replace="..."):
    """
    From "abcdefghijkl1234567" make "ab...67".
    Beginning of string takes precendence (str_vacuum("123",2,'.') -> "1.")
    @param string: The string to look at.
    @param max_length: If length of input string is shorter than max_length,
                       simply return input string.
    @param replace: What to replace vacuum-cleaned stuff with.
    @return: String which has been vacuum-cleaned if too long.
    """
    len_input = len(string)

    if len_input <= max_length:
        return string

    if len(replace) +1 > max_length:
        # TODO make this a valueerror
        raise Exception()

    len_r = len(replace)
    nr_cuts = (len_input + len_r - max_length)
    output = string
    stpos = int((len_input - nr_cuts)/2.0 + 0.5)
    epos = int((len_input - nr_cuts)/2.0 )
    output = string[0:stpos] + replace
    if epos != 0:
        output += string[-epos:]
    return output


