# -*- coding: utf-8 -*-

# mapWOC
#
# Description: Process to poll a number of Nodes.
#
# Authors:
# Felix Wolfsteller <felix.wolfsteller@intevation.de>
#
# Copyright:
# Copyright (C) 2011 Bundesamt für Sicherheit in der Informationstechnik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2,
# or, at your option, any later version as published by the Free
# Software Foundation
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

from mapwoc.lib.tickprocess import TickProcess

from nodeconnection import NodeConnection

class NodePollProcess(TickProcess):
    """
    Ask a number of nodes every poll_interval seconds about their state.
    The answers are put in the Queue (as dicts).
    """

    def __init__(self, queue, nodes, poll_interval=60):
        """
        Sets up (but doesnt start) the Process to periodically check for
        nodes state.

        @param nodes: A dict with nodename and NodeConnection instance.
        @param queue: A Queue that will fill up with answers (strings) of the
                      redirector.
        @param poll_interval: The interval in seconds to wait between two
                              communication attempts to the redirector.
        """

        TickProcess.__init__(self, queue, poll_interval)
        self.nodes = nodes

    def tick(self):
        """
        Ask the nodes about their state, put answer string in dict
        in queue, sleep given number of seconds.
        """

        # TODO try/catch here instead of in ask_redirector
        # TODO proper error handling
        nodestates = {}
        for key,node in self.nodes.items():
            try:
                nodestates[key] = node.ask_decode('state')
            except:
                nost = {}
                nost[''] = 'connection broken'
                nodestates[key] = nost
        self.put(nodestates)
