# -*- coding: utf-8 -*-

# mapWOC
#
# Description: KVM-Process related classes.
#
# Authors:
# Felix Wolfsteller <felix.wolfsteller@intevation.de>
#
# Copyright:
# Copyright (C) 2011 Bundesamt für Sicherheit in der Informationstechnik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2,
# or, at your option, any later version as published by the Free
# Software Foundation
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

import subprocess

def start_mitmdump(port, dumpfile, rqscript=None, address=None):
    """
    Starts a mitmdump subprocess.
    @param port: Port for mitmdump on which to listen.
    @param dumpfile: Output file for mitmdump.
    @param rqscript: Request-script to use, if any.
    @param address: Address to listen on.
    @return: Pair (cmd, Popen).
    """
    cmd = ['mitmdump',
           '-p', str(port), # port
           '-w', dumpfile,  # output
           '--cert-wait-time', str(3.0), # sleep after cert-generation
           '-q', # be quiet
          ]
    if rqscript:
        cmd += ['-s', rqscript]
    if address:
        cmd += ['-a', address]
    subp = subprocess.Popen(cmd, close_fds=True)
    return (cmd, subp)

