import logging
from time import strftime

from pylons import request, response, session, tmpl_context as c, url
from pylons.controllers.util import abort, redirect, redirect_to
from pylons.i18n.translation import _, ungettext

import formencode
from pylons.decorators import validate
from pylons.decorators.rest import restrict

import webhelpers.paginate as paginate

from mapwoc.lib.url import URLList, URL, create_URL_List
from mapwoc.lib.scanconfiguration import ScanConfiguration
from mapwoc.lib.scanconfiguration import TransportableSC
from mapwoc.lib.scanconfiguration import VisitorConfiguration
from mapwoc.lib.scanconfiguration import VisitorBundle
from mapwoc.lib.trivialresponse import TrivialResponse
from mapwoc.nodectrl.hcinteractionscript import eventmap_from_string

from mapwocgui.lib.form_transformers import scanconfig_from_form
from mapwocgui.lib.auth import checkAccess
import mapwocgui.model.masterproxy as master
import mapwocgui.lib.helpers as h
from mapwocgui.lib.base import BaseController, render
from mapwocgui.lib.forms import ManualScanForm
from mapwocgui.lib.forms import ScanForm
from mapwocgui.lib.forms import QuickScanForm
from mapwocgui.lib.forms import TimedScanForm
from mapwocgui.model import Session


log = logging.getLogger(__name__)

# TODO initially, no URL Lists exist, thus no scan can be created! Fix that
# TODO move validators to lib

# TODO Validate that the QuickScan List is not used except for QuickScans.

def get_add_scan(scan_name):
    """Gets the first ScanConfiguration by name, create one and add to
       session if it does not exist."""
    scan = Session.query(ScanConfiguration).filter_by(name=scan_name).first()
    if not scan:
        scan = ScanConfiguration(scan_name)
        Session.add(scan)
    return scan

# TODO Further outsource this function to general integrity check/setup.
def get_quickscan():
    """Gets the QuickScan ScanConfiguration, creating it if it does not exist."""

    quickscan = get_add_scan("QuickScan")

    if not quickscan.visitor_configuration:
        qs_vc = VisitorConfiguration()
        Session.add(qs_vc)
        quickscan.visitor_configuration = qs_vc

    if not quickscan.visitor_configuration.visitors[0]:
        quickscan.visitor_configuraiton.visitors.append(VisitorBundle("",""))
        Session.add(quickscan.visitor_configuration.visitors[0])

    quickscan_urllist = Session.query(URLList).filter_by(name="QuickScan").first()
    if not quickscan_urllist:
        quickscan_urllist = URLList("QuickScan")
        quickscan_urllist.numurls = 1
        Session.add(quickscan_urllist)
        quickscan.urlcollection = [quickscan_urllist]

    if not quickscan.urlcollection[0]:
        quickscan_urllist = URLList("QuickScan")
        quickscan_urllist.numurls = 1
        Session.add(quickscan_urllist)
        quickscan.urlcollection = [quickscan_urllist]

    quickscan.visitor_configuration.eventmap = "10h"
    quickscan.visitor_configuration.active_window = 60

    return quickscan

def get_manualscan():
    """Gets the ManualScan ScanConfiguration, creating it if it does not exist."""
    manualscan = get_add_scan("ManualScan")

    if not manualscan.visitor_configuration:
        qs_vc = VisitorConfiguration()
        Session.add(qs_vc)
        manualscan.visitor_configuration = qs_vc

    manualscan.visitor_configuration.eventmap = ""

    return manualscan

def get_timedscan():
    """Gets the TimedScan ScanConfiguration, creating it if it does not exist."""

    timedscan = get_add_scan("TimedScan")

    if not timedscan.visitor_configuration:
        qs_vc = VisitorConfiguration()
        Session.add(qs_vc)
        timedscan.visitor_configuration = qs_vc

    timedscan.visitor_configuration.eventmap = ""

    return timedscan

class ScanController(BaseController):

    def index(self):
        scanconfigurations = Session.query(ScanConfiguration)
        c.scanconfigurations_paginator = paginate.Page(scanconfigurations,
                                                       page=int(
                                               request.params.get('page', 1)),
                                                       items_per_page=20,)
        c.hcs = master.get_hcs()
        c.available_vms = []
        for hc in c.hcs:
            if hc.hctype != 'native':
                for node in hc.on_nodes:
                    c.available_vms.append(hc.hcsid + ", " + node)
        c.current_scan_name = master.get_state_set().get('scanname')
        c.native_hcs = [hc for hc in c.hcs if hc.hctype == 'native']
        return render('/scan/index_scan.mako')

    def view(self, id):
        c.scanconfig = Session.query(ScanConfiguration).get(int(id))
        c.scanconfig.scanruns.sort(key=lambda scanrun:scanrun.starttime, reverse=True)
        hcs = master.get_hcs()
        c.master_state = master.get_state_set()
        c.hcs = {}
        for hc in hcs:
            c.hcs[hc.hcsid] = hc
        c.scanrun_paginator = paginate.Page(c.scanconfig.scanruns,
                                     page=int(request.params.get('page', 1)),
                                     items_per_page=10,)
        c.urllist_paginator = paginate.Page(c.scanconfig.urlcollection,
                                     page=int(request.params.get('upage',1)),
                                     items_per_page=30,)
        c.eventmap = eventmap_from_string(c.scanconfig.visitor_configuration.eventmap, None)
        return render('/scan/view_scan.mako')

    @checkAccess("privileged")
    def create(self):
        all_urllists = Session.query(URLList).filter_by(deleted=False).all()
        c.available_urllists = [(u.id, u.name) for u in all_urllists]

        if not c.available_urllists:
            h.flash(_(u"No Urllists avaiable to create scan"))

        c.hcs = master.get_hcs()
        return render('scan/create_scan.mako')

    # TODO not checkAccess?
    def create_quickscan(self):
        c.hcs = master.get_hcs()
        c.available_vms = []
        for hc in c.hcs:
            if hc.hctype != 'native':
                for node in hc.on_nodes:
                    c.available_vms.append(hc.hcsid + ", " + node)
        c.current_scan_name = master.get_state_set().get('scanname')
        c.native_hcs = [hc for hc in c.hcs if hc.hctype == 'native']
        c.available_vms = sorted(c.available_vms)
        return render('scan/create_quick_scan.mako')

    # TODO not checkAccess?
    def create_manualscan(self):
        c.hcs = master.get_hcs()
        c.available_vms = []
        for hc in c.hcs:
            if hc.hctype != 'native':
                for node in hc.on_nodes:
                    c.available_vms.append(hc.hcsid + ", " + node)
        c.current_scan_name = master.get_state_set().get('scanname')
        c.native_hcs = [hc for hc in c.hcs if hc.hctype == 'native']
        c.available_vms = sorted(c.available_vms)
        return render('scan/create_manual_scan.mako')

    # TODO Should be post_only=True?
    @checkAccess("privileged")
    @validate(schema=ScanForm(), form='create', post_only=False, on_get=True)
    def new(self):
        # Create the ScanConfig and register it in database.
        sc = scanconfig_from_form(self.form_result)

        Session.add(sc)
        Session.commit()

        redirect_to(controller='scan', action='view', id=sc.id)

    @checkAccess("privileged")
    def delete(self, id):
        scanc = Session.query(ScanConfiguration).filter_by(id=id).first()
        if scanc is None:
            abort(404)
        Session.delete(scanc)
        Session.commit()

        redirect_to(controller='scan', action='index')

    # TODO checkAccess?
    def list(self):
        c.scanconfigurations = Session.query(ScanConfiguration).all()
        return render('/scan/list_scans.mako')

    @checkAccess("privileged")
    def start(self, id):
        """Start a scan config."""

        scanconfig = Session.query(ScanConfiguration).get(int(id))

        response = master.start_scan(TransportableSC(scanconfig))

        if response.error == 0:
            redirect_to(controller='state',action='index')
        else:
            h.flash(response.message)
            redirect_to(controller='scan', action='index')

    @checkAccess("privileged")
    def stop_smooth(self):
        """ Stops any running scan. """
        master.stop_smooth()
        # TODO fetch feedback
        redirect_to(controller='state',action='index')

    @checkAccess("privileged")
    def stop(self):
        """ Stops any running scan. """
        master.stop_scan()
        # TODO fetch feedback
        redirect_to(controller='state',action='index')

    @checkAccess("privileged")
    @validate(schema=QuickScanForm(), form='index', post_only=False, on_get=True)
    def quickscan(self):
        qsc = get_quickscan()

        qsc.comment = _(u"Last changed: %s") % strftime(_("%Y-%m-%d-%H-%M-%S"))

        vc = qsc.visitor_configuration
        vc.visitors[0].node = self.form_result['vm'].split(", ")[1]
        vc.visitors[0].number = 1
        vc.visitors[0].hcsid = self.form_result['vm'].split(", ")[0]

        url_str = self.form_result['quickurl']
        url = Session.query(URL).filter_by(url=url_str).first()

        # Add the URL if it is not yet in database
        if not url:
            url = URL(url_str)
            Session.add(url)

        qsc.urlcollection[0].comment = qsc.comment
        qsc.urlcollection[0].urls = [url]

        # Enable VNC?
        if self.form_result['enable_vnc']:
            qsc.enable_vnc = True
        else:
            qsc.enable_vnc = False

        Session.add(qsc)
        Session.commit()

        return self.start(qsc.id)

    @checkAccess("privileged")
    @validate(schema=ManualScanForm(), form='index', post_only=False, on_get=True)
    def manualscan(self):
        manual_scan = get_manualscan()

        # Get time from form. Its preconfigured choices so we know its int.
        timef = int(self.form_result['time_frame']) * 60
        manual_scan.visitor_configuration.active_window = -timef

        manual_scan.comment = _(u"Last changed: %s") % strftime(_("%Y-%m-%d-%H-%M-%S"))

        vc = manual_scan.visitor_configuration
        vb = VisitorBundle(self.form_result['vm'].split(", ")[0], 1, self.form_result['vm'].split(", ")[1])
        if len(vc.visitors) == 0:
            vc.visitors.append(vb)
        else:
            vc.visitors[0].node = self.form_result['vm'].split(", ")[1]
            vc.visitors[0].number = 1
            vc.visitors[0].hcsid = self.form_result['vm'].split(", ")[0]

        url_str = self.form_result['manualurl']
        url_list_str = self.form_result['manualurllist']
        #url = Session.query(URL).filter_by(url=url_str).first()
        # TODO same for URLLIST

        # Add the URL if it is not yet in database
        #if not url:
        #    url = URL(url_str)
        #    Session.add(url)

        #manual_scan.urlcollection[0].comment = qsc.comment
        #manual_scan.urlcollection[0].urls = [url]

        manual_scan.enable_vnc = True
        Session.add(manual_scan)
        Session.commit()

        # TODO other views (and probably the message) need now
        #      to react, show the VNC port and allow stop.

        return self.start(manual_scan.id)


    @checkAccess("privileged")
    @validate(schema=TimedScanForm(), form='timedscan', post_only=False, on_get=True)
    def start_timed(self):
        # Get TimedScan Configuration.
        timedscan = get_timedscan()

        # Reset visitors.
        # TODO delete "old visitors"?
        vc = timedscan.visitor_configuration
        vc.visitors = []

        # Get HCs from form
        for k in self.form_result:
            if k.startswith("hc") and self.form_result[k] == '1':
                node = k.split(",")[1]
                vc.visitors.append(VisitorBundle(k[2:].split(",")[0],
                                                 1, node))

        # Get time from form.
        timef = self.form_result['time_frame'] * 60
        timedscan.visitor_configuration.active_window = -timef

        # Get URL Lists from form.
        urls = self.form_result.get('urllist')
        timedscan.urlcollection = []
        if urls:
            for selected in self.form_result['urllist']:
                timedscan.urlcollection += [Session.query(URLList).get(int(selected))]

        # Add to Session.
        Session.add(timedscan)
        timedscan.comment = _(u"Last changed: %s") % strftime(_("%Y-%m-%d-%H-%M-%S"))
        Session.commit()

        # Start the thing
        return self.start(timedscan.id)

    @checkAccess("privileged")
    def timedscan(self):
        # Get all the data needed for a timed scan.
        c.hcs = [hc for hc in master.get_hcs() if hc.hctype == 'native']
        c.hcs = sorted(c.hcs, key=lambda hc: hc.name)
        all_urllists = Session.query(URLList).filter_by(deleted=False).all()
        c.available_urllists = [(u.id, u.name) for u in all_urllists]
        return render('/scan/create_timed_scan.mako')
