import logging
import ConfigParser
from StringIO import StringIO
import datetime

from pylons import request, response, session, tmpl_context as c, url
from pylons.controllers.util import abort, redirect

from mapwocgui.lib.base import BaseController, render
from mapwocgui.model import Session
from mapwoc.lib.scanconfiguration import ScanConfiguration
from mapwoc.lib.url import URL
import mapwocgui.model.masterproxy as master
from mapwocgui.controllers.url import screendump

log = logging.getLogger(__name__)

class StateController(BaseController):

    def index(self):
        c.masterproxy_state_set = master.get_state_set()
        c.master_version = master.get_version()
        c.nodestates = c.masterproxy_state_set.get('nodes')

        c.hasvncs = False
        for nodename,state in c.nodestates.items():
            # Any VNC-Port listed?
            for [hc,vncport] in state.get('runlist'):
                if vncport:
                    c.hasvncs = True
                    break;

        redirectorstate = c.masterproxy_state_set.get('redirector')
        if not redirectorstate:
            redirectorstate = {'state' : 'unreachable', 'last_urls' : [],
                               'next_urls' : ['n/a'], 'n_urls' : 0,
                               'n_urls_served' : 0, 'n_last_urls' : 0,
                               'n_next_urls' : 0,
                               'version' : 'n/a'}

        c.redirectorstate = redirectorstate.get('state')
        c.redirector_last_urls = redirectorstate.get('last_urls')
        c.redirector_last_urls_screens = []
        c.redirector_last_urls_ids = []
        for lurl in c.redirector_last_urls:
            urlobj = Session.query(URL).filter_by(url=lurl.url).first()
            c.redirector_last_urls_screens.append(screendump(urlobj.id, lurl.hcsid))
            c.redirector_last_urls_ids.append(urlobj.id)

        next_urls = redirectorstate.get('next_urls')
        c.redirector_next_urls = []
        for nurl in next_urls:
            screendumpurl = Session.query(URL).filter_by(url=nurl).first()
            if screendumpurl == None: continue # Can happen after error
            [_, screen_url] = screendump(screendumpurl.id, "")
            c.redirector_next_urls.append([screendumpurl, screen_url])

        c.current_scan = c.masterproxy_state_set.get('scanname')
        if c.current_scan:
            c.current_scan_id = Session.query(ScanConfiguration).filter_by(
                    name=c.current_scan).first().id

        c.hcs = master.get_hcs()
        # Sort by name
        c.hcs = sorted(c.hcs, key=lambda hc: hc.name)

        return render('/state/index_state.mako')
