import logging

import glob
from os import path
from paste.fileapp import FileApp
from routes import url_for

from pylons import config, request, response, session, tmpl_context as c, url
from pylons.controllers.util import abort, redirect

import webhelpers.paginate as paginate

from mapwocgui.lib.base import BaseController, render
from mapwocgui.model import Session

from mapwoc.lib.result import Result, Visit
from mapwoc.lib.url import URL, URLList

log = logging.getLogger(__name__)

# TODO move to gui/lib
def screendump(urlid, hcsid):
    """
    Returns [hcsid1, url] to screendump.
    hcsid1 will be the "best" match if no screendump
    for given hcsid was found.
    """

    work_dir = path.dirname(config.get("sqlalchemy.url").lstrip("sqlite:"))
    filename = path.join(work_dir, "screendumps", str(urlid))
    # Iterate over these files, skip the .png part of it.
    screendump_files = glob.glob(filename + "/*")
    screendump_hcs = [ path.basename(img)[0:-4] for img in screendump_files ]

    # Now, is there a 'real' match?
    if hcsid in screendump_hcs:
        return [hcsid, url_for(controller="url", action="screendump", id=urlid, hcsid=hcsid+"/")]

    if len(screendump_hcs) == 0:
        return [None,None]
    elif len(screendump_hcs) == 1:
        return [screendump_hcs[0], url_for(controller="url", action="screendump", id=urlid, hcsid=screendump_hcs[0]+"/")]


    # Search next best option
    scores = {}
    for shcsid in screendump_hcs:
        score = 0
        for i in range(0,min(len(hcsid),len(shcsid))):
            if hcsid[i] == shcsid[i]:
                score = score + 1
            else:
                break
        scores[score] = shcsid

    # Take highest score
    best_hcsid = scores[sorted(scores.keys())[-1]]
    return [best_hcsid,url_for(controller="url", action="screendump", id=urlid, hcsid=best_hcsid+"/")]

def screendumps(urlid):
    # Get the list of hcs with screendumps.
    work_dir = path.dirname(config.get("sqlalchemy.url").lstrip("sqlite:"))
    filename = path.join(work_dir, "screendumps", str(urlid))
    # Iterate over these files, skip the .png part of it.
    screendumps = glob.glob(filename + "/*")
    screendump_hcs = [ path.basename(img)[0:-4] for img in screendumps ]
    return screendump_hcs


class UrlController(BaseController):

    def index(self):
        url_query = Session.query(URL)
        c.url_paginator = paginate.Page(url_query,
                                        page=request.params.get('page', 1),
                                        items_per_page=40,)
        return render('/url/index_url.mako')

    def view(self, id):
        c.url = Session.query(URL).get(int(id))
        visits = Session.query(Visit).filter(Visit.urls.any(id=id))
        c.visit_paginator = paginate.Page(visits,
                              page=request.params.get('rpage', 1),
                              items_per_page=40,
                              )
        c.urllist_paginator = paginate.Page(
                              c.url.urllist,
                              page=request.params.get('upage', 1),
                              items_per_page=10,
                              )
        # Get the list of hcs with screendumps.
        work_dir = path.dirname(config.get("sqlalchemy.url").lstrip("sqlite:"))
        filename = path.join(work_dir, "screendumps", str(id))
        # Iterate over these files, skip the .png part of it.
        screendumps = glob.glob(filename + "/*")
        c.screendump_hcs = [ path.basename(img)[0:-4] for img in screendumps ]
        return render('/url/view_url.mako')

    def screendump(self, id, hcsid="unset"):
        """
        Gets an image. If hcsid is unset, takes first image for given (URL) id.
        """
        work_dir = path.dirname(config.get("sqlalchemy.url").lstrip("sqlite:"))
        filename = path.join(work_dir,"screendumps", str(id), str(hcsid)+".png")
        # If hcsid is unset, get the first png from that folder.
        if hcsid == "unset":
            work_dir = path.dirname(config.get("sqlalchemy.url").lstrip("sqlite:"))
            filename = path.join(work_dir, "screendumps", str(id))
            filename = glob.glob(filename + "/*")[0]

        file_size = path.getsize(filename)

        user_filename = "screendump.png"
        headers = [('Content-Disposition', 'filename=\"' + \
                        user_filename + '\"'),
                   ('Content-Type', 'image/png'),
                   ('Content-Length', str(file_size))]
        fapp = FileApp(filename, headers=headers)

        return fapp(request.environ, self.start_response)
