from mapwocgui.lib.base import render
import mapwocgui.lib.helpers as h
from pylons import session
from pylons.i18n.translation import _
from pylons import request
from decorator import decorator
from webob.exc import HTTPForbidden

def render_signin():
    if not request.environ.get('REMOTE_USER'):
        if request.params:
            h.flash(_(u"Invalid username or password"))
        return render('account/signin.mako')
    else:
        session['login_tries'] = None
        session['REMOTE_USER'] = request.environ.get('REMOTE_USER')
        session.save()
        return render('/account/signedin.mako')

def checkAccess(role, allow_admin = True):
    """ Raises an HTTPUnauthorized if the user does not have the role role
        By default admins are always allowed

        @param role: The role Required for acces
        @param allow_admin: Allow only users of role role and not admins
    """
    def validate(func, self, *args, **kwargs):
        user = request.environ.get('REMOTE_USER')
        # Admins can do everything
        if allow_admin and request.environ['authkit.users'].user_has_role(user, "admin"):
            return func(self, *args, **kwargs)
        elif request.environ['authkit.users'].user_has_role(user, role):
            return func(self, *args, **kwargs)
        else:
            raise HTTPForbidden(explanation=_('You are not authorised to '
                'look at the requested document. At least "%s" access '
                'is required.' % role_to_local_string(role)))
    return decorator(validate)

def user_set_single_role(user, role):
    """ Remove all other roles and only set the selected role for the user"""
    if not h.authorized(h.is_admin):
        # This should never happen and be checked and handled
        # before user set roles is called
        return
    authkit_users = request.environ['authkit.users']

    for ro in ["privileged", "admin"]:
        if authkit_users.user_has_role(user, ro):
            authkit_users.user_remove_role(user, ro)

    if role != "none":
        authkit_users.user_add_role(user, role)

def role_to_local_string(role):
    """ Returns a localized string representation of the role name role

        @param role: The role name as used in authkit
        @returns A localized string for the role name as used in the gui
    """
    if role == "admin":
        return _(u"Administrator")
    if role == "privileged":
        return _(u"Normal User")
    return _(u"Visitor")


def role_for_user(user_name):
    """ Returns a localized string representation of the highest role for
        user user_name

        @param user_name: The Username as used in authkit
        @returns A localized string for the role name as used in the gui
    """
    if request.environ['authkit.users'].user_has_role(user_name, 'admin'):
        return role_to_local_string("admin")
    if request.environ['authkit.users'].user_has_role(user_name, 'privileged'):
        return role_to_local_string("privileged")
    return role_to_local_string("none")

