"""The base Controller API

Provides the BaseController class for subclassing.
"""
from pylons.controllers import WSGIController
from pylons.templating import render_mako as render
from pylons import session, tmpl_context as c
from pylons import request
from pylons import config
from pylons.i18n.translation import set_lang, _
from pylons.controllers.util import redirect_to, abort
from authkit.permissions import ValidAuthKitUser
from time import time, ctime
import mapwocgui.lib.helpers as h
from mapwoc.lib.users import UserData
from mapwocgui.model.meta import Session

import mapwocgui.model.masterproxy as master

class BaseController(WSGIController):

    # TODO need some intermediate before/after
    def __before__ (self, action, **params):
        # This is actually called after the __call__ function
        # but before the wsgi controller's action
        c.messages = master.get_messages()
        if request.environ.get('REMOTE_USER'):
            user = Session.query(UserData).filter_by(
                    user_name=request.environ.get('REMOTE_USER').lower()).first()
            session['lang'] = user.language

        if 'lang' in session:
            set_lang(session['lang'])

    def __call__(self, environ, start_response):
        """Invoke the Controller"""
        # WSGIController.__call__ dispatches to the Controller method
        # the request is routed to. This routing information is
        # available in environ['pylons.routes_dict']
        if not environ.get('REMOTE_USER'):
            abort(401)

        if not session.get('REMOTE_USER'):
            if config.get("debug") != True:
                # Disabled for debugging
                session.clear()
                session.save()
                request.environ['paste.auth_tkt.logout_user']()
                # Logout does not remove the user from the environ
                request.environ['REMOTE_USER'] = None
                h.flash(_(u"Your session has expired. Please sign in again."))
                abort(401)

        try:
            return WSGIController.__call__(self, environ, start_response)
        except ValueError: 
            redirect_to(controller='canttalkto',action='master')
        finally:
            Session.remove()
