# -*- coding: utf-8 -*-

# mapWOC
#
# Description: Helper functions to handle CPE strings
#
# Authors:
# Andre Heinecke <aheinecke@intevation.de>
#
# Copyright:
# Copyright (C) 2011 Bundesamt für Sicherheit in der Informationstechnik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2,
# or, at your option, any later version as published by the Free
# Software Foundation
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

import mapwocgui.model.masterproxy as master
import re

ICONDICT = {
"debian_linux" : "/img/os_debian.png",
"firefox"      : "/img/browser_firefox.png",
"konqueror"    : "/img/browser_konqueror.png",
"windows_7"    : "/img/os_win7.png",
"internet_explorer" : "/img/browser_internet_explorer.png",
"ie": "/img/internet_explorer.png",
"chrome" : "/img/browser_chrome.png",
"opera_browser" : "/img/browser_opera.png",
"safari" : "/img/browser_safari.png",
"windows_xp" : "/img/os_winxp.png",
"mac_os_x_server" : "/img/os_apple.png",
"windows_vista" : "/img/os_winvista.png",
"ie" : "/img/browser_internet_explorer.png"
}

def os_img_for_hc(hc):
    """
    Returns the filename of the OS icon for hc.
    """
    osexp = re.compile(r"(cpe:/)(o:)([\s\w]+):([\s\w]+)(.*)")
    if not hc:
        return "notfound"
    for cpe in hc.installed_software:
        match = osexp.match(cpe)
        if match:
            osname = match.group(4)
            break
    else:
        return "notfound"
    iconpath = ICONDICT.get(osname)
    if not iconpath:
        return "notfound"
    else:
        return iconpath
    

def os_img_for_hcsid(hcsid):
    """
    Returns the filename of the OS icon for hcsid.
    """
    hc = master.get_hc(hcsid)
    return os_img_for_hc(hc)

def browser_img_for_hcsid(hcsid):
    """
    Returns the filename of the Browser icon for hcsid.
    """
    hc = master.get_hc(hcsid)
    return browser_img_for_hc(hc)

def browser_img_for_hc(hc):
    """
    Returns the filename of the Browser icon for hc.
    """
    browserexp = re.compile(r"(cpe:/)(a:)([\s\w]+):([a-zA-Z_]+)(.*)")
    if not hc:
        return "notfound"
    match = browserexp.match(hc.active_browser)
    if match:
        browsername = match.group(4)
    else:
        return "notfound"
    iconpath = ICONDICT.get(browsername)
    if not iconpath:
        print "no iconpath for %s" % browsername
        return "notfound"
    else:
        return iconpath
