# -*- coding: utf-8 -*-

# mapWOC
#
# Description: From Forms to lib-objects.
#
# Authors:
# Andre Heinecke <andre.heinecke@intevation.de>
# Felix Wolfsteller <felix.wolfsteller@intevation.de>
#
# Copyright:
# Copyright (C) 2011,2012 Bundesamt für Sicherheit in der Informationstechnik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2,
# or, at your option, any later version as published by the Free
# Software Foundation
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

from mapwocgui.model.meta import Session, Base, metadata

from mapwoc.lib.url import URL, URLList
from mapwoc.lib.scanconfiguration import ScanConfiguration
from mapwoc.lib.scanconfiguration import VisitorConfiguration
from mapwoc.lib.scanconfiguration import VisitorBundle

def scanconfig_from_form(form_result):
    sc = ScanConfiguration(form_result['name'])
    sc.comment = form_result['comment']
    vc = VisitorConfiguration()
    vc.active_window = form_result['active_window']
    if form_result.get('fastscan'):
        vc.num_urls = form_result['num_urls']
    sc.visitor_configuration = vc
    if form_result.get('enable_vnc'):
        sc.enable_vnc = True
    else:
        sc.enable_vnc = False
    if form_result['eventmap'] and len(form_result['eventmap']):
        vc.eventmap = form_result['eventmap']
    else:
        vc.eventmap = '10h'

    # form_result['urllist'] is a string if single item, list otherwise.
    fr_urllist = form_result['urllist']
    if isinstance(fr_urllist, list):
        for selected in form_result['urllist']:
            sc.urlcollection += [Session.query(URLList).get(int(selected))]
    elif len(fr_urllist) > 0:
        sc.urlcollection += [Session.query(URLList).get(int(fr_urllist))]

    for k in form_result:
        if k.startswith("hc") and int(form_result[k]) != 0:
            node = k.split(",")[1]
            vc.visitors.append(VisitorBundle(k[2:].split(",")[0],
                                             int(form_result[k]), node))

    return sc

