#!/usr/bin/python

"""Add a user for the mapwocgui."""
import logging

import sys

import paste
import pylons
import pylons.test

from hashlib import md5
from authkit.users.sqlalchemy_driver import UsersFromDatabase

from mapwocgui.config.environment import load_environment
from mapwocgui.model.meta import Session, Base, metadata
import mapwocgui.model as model

from mapwoc.lib.model.metatable import meta_table
from mapwoc.lib.result import ResultType, RESULT_TYPES
from mapwoc.lib.scanconfiguration import ScanConfiguration, VisitorConfiguration, VisitorBundle
from mapwoc.lib.url import URL, URLList
from mapwoc.lib.users import UserData

log = logging.getLogger(__name__)

# Increment this number on every Database change
DB_VERSION = 6

def add_user_noninteractive(conf, username, password):
    """Database and pylons initialization."""

    # Don't reload the app if it was loaded under the testing environment.
    if not pylons.test.pylonsapp:
        load_environment(conf.global_conf, conf.local_conf)

    # Add users to the model (has to be done before db creation).
    log.info("Adding the AuthKit model...")
    users = UsersFromDatabase(model)

    # Create the tables if they don't already exist.
    log.info("Creating db")
    Base.metadata.create_all(bind=Session.bind)

    # Tag with version.
    log.info("Database Version is %d " % DB_VERSION)
    stmt = meta_table.insert(bind=Session.bind)
    stmt.execute(version=DB_VERSION)

    # Non-interactively create first admin user.
    users.user_create(username, password=md5(password).hexdigest())
    users.user_add_role(username, role="admin")
    users.user_add_role(username, role="privileged")
    user_data = UserData(username.lower())
    Session.add(user_data)
    Session.commit()

    print("Thank you, the admin account for %s is now set up." % username)

if __name__ == "__main__":
    if not len(sys.argv) == 4:
        sys.exit("Invoke mapwocgui-adduser.py <path-to-config> <user> <pass>")
    # TODO accept argument of config file path
    path_to_ini_file = sys.argv[1]
    conf = paste.deploy.appconfig('config:' + path_to_ini_file)
    add_user_noninteractive(conf, sys.argv[2], sys.argv[3])

