"""Setup the mapwocgui application"""
import logging
import sys

import pylons.test
from hashlib import md5
from authkit.users.sqlalchemy_driver import UsersFromDatabase

from mapwocgui.config.environment import load_environment
from mapwocgui.model.meta import Session, Base, metadata
import mapwocgui.model as model

from mapwoc.lib.model.metatable import meta_table
from mapwoc.lib.result import ResultType, RESULT_TYPES
from mapwoc.lib.scanconfiguration import ScanConfiguration, VisitorConfiguration, VisitorBundle
from mapwoc.lib.url import URL, URLList
from mapwoc.lib.users import UserData

log = logging.getLogger(__name__)

# Increment this number on every Database change
DB_VERSION = 7

def setup_fill_db():
    """Enter some fixed values into database."""

    log.info("Adding ResultTypes...")
    for key in RESULT_TYPES:
        rt = ResultType(key, RESULT_TYPES[key][0])
        rt.id = RESULT_TYPES[key][1]
        Session.add(rt)

    log.info("Adding QuickScan...")
    quickscan = ScanConfiguration("QuickScan")
    quickscan.comment = "Configuration reserved for QuickScans."
    visitorc = VisitorConfiguration()
    visitorc.eventmap = "10h"
    visitorc.active_window = 60
    visitorc.num_urls = 1
    visitorb = VisitorBundle("",0,"")
    visitorc.visitors.append(visitorb)
    quickscan.visitor_configuration = visitorc
    quickurl = URL("http://mapwoc.org")
    quickurllist = URLList("QuickScan URL-List")
    quickurllist.urls.append(quickurl)
    quickurllist.numurls = 1
    quickurllist.comment = "URL-List reserved for QuickScans."
    quickscan.urlcollection = [quickurllist]
    Session.add(quickscan)
    Session.add(visitorb)
    Session.add(visitorc)
    Session.add(quickurl)
    Session.add(quickurllist)

    # TODO Consolidate e.g. with controllers/scan.
    # TODO Add TimedScan, too.
    # TODO Consolidate with db_integrity_check.
    log.info("Database set up")

def setup_app(command, conf, vars):
    """Database and pylons initialization."""
    # Don't reload the app if it was loaded under the testing environment.
    if not pylons.test.pylonsapp:
        load_environment(conf.global_conf, conf.local_conf)

    # Add users to the model (has to be done before db creation).
    log.info("Adding the AuthKit model...")
    users = UsersFromDatabase(model)

    # Create the tables if they don't already exist.
    log.info("Creating db")
    Base.metadata.create_all(bind=Session.bind)

    log.info("Adding roles and users...")
    # Privileged users read/write access to scans/urls
    users.role_create("privileged")
    # Admin user read/write access to users
    users.role_create("admin")

    # Tag with version.
    log.info("Database Version is %d " % DB_VERSION)
    stmt = meta_table.insert(bind=Session.bind)
    stmt.execute(version=DB_VERSION)

    setup_fill_db()

    # This is a bit of a hack: allow e.g. piping < /dev/null to skip
    # interactive creation of initial admin user.
    if sys.stdin.isatty():
        # Interactively create first admin user.
        print("------------------------------------------------------------")
        print("| Please enter the credentials for the first admin account.")
        username = raw_input("Username: ")
        password = raw_input("Password: ")
        print("------------------------------------------------------------")
        users.user_create(username, password=md5(password).hexdigest())
        users.user_add_role(username, role="admin")
        users.user_add_role(username, role="privileged")
        user_data = UserData(username.lower())
        Session.add(user_data)
        Session.commit()

        print("Thank you, the admin account for %s is now set up." % username)
    else:
        Session.commit()
        print("Not running an interactive session, you will have to add a")
        print("user with the mapwocgui-adduser script.")

